using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Linq;
using MVR;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using GPUTools.Cloth.Scripts;
using Unity.Collections;
using AssetBundles;
using System.Diagnostics;
using GPUTools.Hair.Scripts.Runtime.Render;
using Battlehub.RTCommon;

namespace Stopper {
	public class LightLayerEditor : MVRScript {

        Dictionary<GameObject, int> originalLayers = new Dictionary<GameObject, int>();

        List<JSONStorableFloat> meshRendererFloats = new List<JSONStorableFloat>();

		public override void Init() {
            if(containingAtom.GetComponentInChildren<DAZMergedSkinV2>() != null) {
                AddSlider("Skin Layer", 28, 0, 31, (layer) => {
                    foreach(var skin in containingAtom.GetComponentsInChildren<DAZMergedSkinV2>()){
                        SaveOriginalLayer(skin);
                        skin.gameObject.layer = (int)layer;
                    }
                });
            }

            if(containingAtom.GetComponentInChildren<HairRender>() != null) {
                AddSlider("Hair Layer", 28, 0, 31, (layer) => {
                    foreach(var hairRender in containingAtom.GetComponentsInChildren<HairRender>()){
                        SaveOriginalLayer(hairRender);
                        hairRender.gameObject.layer = (int)layer;
                    }
                });
            }

            AddSlider("All Mesh Renderers", 0, 0, 31, (layer) => {
                foreach(var meshRendererFloat in meshRendererFloats) {
                    meshRendererFloat.val = layer;
                }
            }, true);

            foreach(var meshRenderer in containingAtom.GetComponentsInChildren<MeshRenderer>()){
                if(meshRenderer.enabled) {
                    meshRendererFloats.Add(AddSlider(meshRenderer.gameObject.name, meshRenderer.gameObject.layer, 0, 31, (layer) => {
                        SaveOriginalLayer(meshRenderer);
                        meshRenderer.gameObject.layer = (int) layer;
                    }, true));
                }
            }
		}

        public JSONStorableFloat AddSlider(string name, float defaultVal, float min, float max, JSONStorableFloat.SetFloatCallback onChange = null, bool rightSide = false){
            JSONStorableFloat storableFloat = new JSONStorableFloat(name, defaultVal, min, max, true, true);
            storableFloat.storeType = JSONStorableParam.StoreType.Full;
            UIDynamicSlider uiSlider = this.CreateSlider(storableFloat, rightSide);
            uiSlider.rangeAdjustEnabled = false;
            uiSlider.slider.wholeNumbers = true;
            RegisterFloat(storableFloat);
            
            if(onChange != null){
                storableFloat.setCallbackFunction = onChange;
            }

            onChange(defaultVal);
            
            return storableFloat;
	    }

        private void SaveOriginalLayer(Component component) {
            if(!originalLayers.ContainsKey(component.gameObject)){
                originalLayers[component.gameObject] = component.gameObject.layer;
            }
        }
		
        private string GetPathToRoot(GameObject go){
            if (go.transform.parent == null || go.transform.parent == containingAtom.transform)
                return "/" + go.name;
            return GetPathToRoot(go.transform.parent.gameObject) + "/" + go.name;
        }

        private void OnEnable() {
            foreach(var storableFloatName in GetFloatParamNames()){
                GetFloatJSONParam(storableFloatName).setCallbackFunction(GetFloatJSONParam(storableFloatName).val);
            }
        }
		
		private void OnDisable()
		{
			foreach(var entry in originalLayers) {
                if(entry.Key != null) {
                    SuperController.LogMessage($"restoring {entry.Key} to {entry.Value}");
                    entry.Key.layer = entry.Value;
                }
            }
		}
	}
}